@@ -1,6 +1,6 @@
 =head1 NAME
 
-Spreadsheet::Write - simplified writer for spreadsheet files
+Spreadsheet::Write - Simplified writer for CSV or XLS (MS Excel'97) files
 
 =head1 SYNOPSIS
 
@@ -23,6 +23,7 @@ Spreadsheet::Write - simplified writer for spreadsheet files
         style           => 'money',
         font_weight     => 'bold',
         font_color      => 42,
+        bg_color        => 'gray',
         font_face       => 'Times New Roman',
         font_size       => 20,
         align           => 'center',
@@ -46,12 +47,10 @@ Spreadsheet::Write - simplified writer for spreadsheet files
 
 =head1 DESCRIPTION
 
-C<Spreadsheet::Write> writes files in CSV, Microsoft Excel,
-HTML and OpenDocument formats. It is especially suitable
-for building various dumps and reports where rows are built
-in sequence, one after another.
-
-It is not especially suitable for modifying existing files.
+C<Spreadsheet::Write> writes files in CSV, XLS (Microsoft Excel 97),
+and potentially other formats. It is especially suitable for building
+various dumps and reports where rows are built in sequence, one after
+another.
 
 =head1 METHODS
 
@@ -60,19 +59,23 @@ It is not especially suitable for modifying existing files.
 ###############################################################################
 package Spreadsheet::Write;
 
-our $VERSION = '0.100';
-
-use 5.008;
-use common::sense;
+require 5.008_001;
 
+use strict;
 use IO::File;
 
-sub version {
-  return $VERSION;
+BEGIN {
+    use vars       qw($VERSION);
+    $VERSION =     '1.0';
 }
+
+sub version {
+    return $VERSION;
+}
+
 ###############################################################################
 
-=head2 new()
+=head2 new ()
 
     $spreadsheet = Spreadsheet::Write->new(
         file            => 'table.xls',
@@ -84,78 +87,85 @@ sub version {
 Creates a new spreadsheet object. It takes a list of options. The
 following are valid:
 
-    file        filename of the new spreadsheet (mandatory)
+    file        filename of the new spreadsheet or an IO handle (mandatory)
     encoding    encoding of output file (optional, csv format only)
-    format      format of spreadsheet - 'csv', 'xls', 'html', 'ods' or 'auto' (default).
+    format      format of spreadsheet - 'csv', 'xls', or 'auto' (default)
     sheet       Sheet name (optional, not supported by some formats)
     styles      Defines cell formatting shortcuts (optional)
-    failsafe    Boolean - if true, falls back to CSV in emergencies
 
 If file format is 'auto' (or omitted), the format is guessed from the
 filename extention. If impossible to guess the format defaults to 'csv'.
 
+An IO-like handle can be given as 'file' argument (IO::File, IO::Scalar,
+etc). In this case the format argument is mandatory.
+
+Default styles are:
+    header => {
+        font_weight     => 'bold',
+        type            => 'string',
+    },
+    ntext => {
+        format          => '@',
+        type            => 'string',
+    },
+    money => {
+        format          => '$#,##0.00;-$#,##0.00',
+    },
+
 =cut
 
-sub new(@)
-{
-	my ($class, %args) = @_;
-	
-	my $format   = $args{'format'} || 'auto';
-	my $filename = $args{'file'}   || $args{'filename'};
-	
-	if (lc $format eq 'auto')
-	{
-		$format = ($filename=~/\.(.+)$/) ? lc($1) : 'auto';
-	}
-	
-	my $implementation = {
-		auto    => 'CSV',
-		csv     => 'CSV',
-		excel   => 'Excel',
-		html    => 'HTML',
-		ods     => 'OpenDocument',
-		odsxml  => 'OpenDocumentXML',
-		text    => 'CSV',
-		txt     => 'CSV',
-		xhtml   => 'XHTML',
-		xls     => 'Excel',
-		xml     => 'OpenDocumentXML',
-		}->{lc $format};
-	
-	my $self = eval
-	{
-		die "Format $format is not supported" unless $implementation;
-		$implementation = join '::', (__PACKAGE__, $implementation);	
-		eval "use $implementation;";
-		die $@ if $@;
-		return $implementation->new(%args);
-	};
-	
-	if ($self && !$@)
-	{
-		return $self;
-	}
-	elsif ($args{'failsafe'})
-	{
-		$implementation = join '::', (__PACKAGE__, 'CSV');
-		eval "use $implementation;";
-		die $@ if $@;
-		return $implementation->new(%args);
-	}
-	elsif ($@)
-	{
-		die $@;
-	}
-	else
-	{
-		die "Could not instantiate spreadsheet!\n";
-	}
+sub new(@) {
+    my $proto = shift;
+    my $args=ref($_[0]) eq 'HASH' ? $_[0] : {@_};
+
+    my $filename=$args->{'file'} || $args->{'filename'} || die 'No file given';
+
+    my $format=$args->{'format'} || 'auto';
+    if($format eq 'auto') {
+        die "Need a 'format' argument for IO-handle in 'file'" if ref $filename;
+        $format=($filename=~/.*\.(.+?)$/) ? lc($1) : 'csv';
+    }
+
+    # Some commonly used styles
+    #
+    my %parm=%{$args};
+    $parm{'styles'}->{'ntext'}||={
+        format          => '@',
+        type            => 'string',
+    };
+    $parm{'styles'}->{'header'}||={
+        font_weight     => 'bold',
+        type            => 'string',
+    };
+    $parm{'styles'}->{'money'}||={
+        format          => '$#,##0.00;-$#,##0.00',
+    };
+
+    # CPAN ownership of Spreadsheet::Write::CSV and a number of other
+    # simpler names belongs to Toby Inkman, with Spreadsheet::Wright
+    # clone.
+    #
+    my $implementation={
+        csv     => 'WriteCSV',
+        xls     => 'WriteXLS',
+    }->{lc $format};
+
+    $implementation ||
+        die "Format $format is not supported";
+
+    $implementation=join('::',(__PACKAGE__,$implementation));
+
+    eval "use $implementation;";
+    die $@ if $@;
+
+    return $implementation->new(%parm);
 }
 
 ###############################################################################
 
 sub DESTROY {
     my $self=shift;
+    ### print STDERR "DESTROY: ".ref($self)."\n";
     $self->close();
 }
 
@@ -168,23 +178,36 @@ sub error {
 
 ###############################################################################
 
-sub _open($)
-{
-	my $self=shift;
+sub _open($) {
+    my $self=shift;
 
-	$self->{'_CLOSED'} && die "Can't reuse a closed spreadsheet";
+    $self->{'_CLOSED'} && die "Can't reuse a closed spreadsheet";
 
-	my $fh = $self->{'_FH'};
+    my $fh=$self->{'_FH'};
 
-	if(!$fh)
-	{
-		my $filename = $self->{'_FILENAME'} || return undef;
-		$fh=new IO::File;
-		$fh->open($filename,"w") || die "Can't open file $filename for writing: $!";
-		$self->{'_FH'}=$fh;
-	}
+    if(!$fh) {
+        my $filename=$self->{'_FILENAME'} || return undef;
 
-	return $self->_prepare;
+        if(ref($filename)) {
+            $fh=$filename;
+            $self->{'_EXT_HANDLE'}=1;
+        }
+        else {
+            $fh=new IO::File;
+            $fh->open($filename,"w") || die "Can't open file $filename for writing: $!";
+            $fh->binmode(':utf8');
+        }
+    }
+
+    $self->{'_FH'}=$fh;
+
+    return $self->_prepare;
+}
+
+###############################################################################
+
+sub _prepare {
+    # no-op by default
 }
 
 ###############################################################################
@@ -192,12 +215,12 @@ sub _open($)
 =head2 addrow(arg1,arg2,...)
 
 Adds a row into the spreadsheet. Takes arbitrary number of
-arguments. Arguments represent column values and may be strings or hash
-references. If an argument is a hash reference, additional optional
-parameters may be passed:
+arguments. Arguments represent cell values and may be strings or hash
+references. If an argument is a hash reference, it takes the following
+structure:
 
-    content         value to put into column
-    style           formatting style, as defined in new()
+    content         value to put into the cell
+    style           formatting style, as defined in new(), scalar or array-ref
     type            type of the content (defaults to 'auto')
     format          number format (see Spreadsheet::WriteExcel for details)
     font_weight     weight of font. Only valid value is 'bold'
@@ -206,9 +229,12 @@ parameters may be passed:
     font_face       font of column; default is 'Arial'
     font_color      color of font (see Spreadsheet::WriteExcel for color values)
     font_size       size of font
+    bg_color        color of background (see Spreadsheet::WriteExcel for color values)
     align           alignment
     valign          vertical alignment
     width           column width, excel units (only makes sense once per column)
+    height          row height, excel units (only makes sense once per row)
+    comment         hidden comment for the cell, where supported
 
 Styles can be used to assign default values for any of these formatting
 parameters thus allowing easy global changes. Other parameters specified
@@ -250,66 +276,69 @@ as:
         }
     );
 
-For CSV format all extra arguments are safely ignored.
+Not all styling options are supported in all formats. Where they are not
+supported they are safely ignored.
 
 =cut
 
-sub addrow (@)
-{
-	my $self = shift;
-	$self->_open() || return undef;
-	
-	my @cells;
-	
-	foreach my $item (@_)
-	{
-		if (ref $item eq 'HASH')
-		{
-			push @cells, $item;
-		}
-		else
-		{
-			push @cells, { content => $item };
-		}
-	}
-	
-	return $self->_add_prepared_row(@cells);	
+sub addrow (@) {
+    my $self = shift;
+    $self->_open() || return undef;
+
+    my @cells;
+
+    foreach my $item (@_) {
+        if (ref $item eq 'HASH') {
+            if (ref $item->{'content'} eq 'ARRAY') {
+                foreach my $i (@{ $item->{'content'} }) {
+                    my %newitem = %$item;
+                    $newitem{'content'} = $i;
+                    push @cells, \%newitem;
+                }
+            }
+            else {
+                push @cells, $item;
+            }
+        }
+        else {
+            push @cells, { content => $item };
+        }
+    }
+
+    return $self->_add_prepared_row(@cells);
 }
 
 ###############################################################################
 
-=head2 addrows([Cell_1A,Cell_1B,...],[Cell_2A,Cell_3B,...],...)
+=head2 addrows([$cell1A,$cell1B,...],[$cell2A,$cell2B,...],...)
 
 Shortcut for adding multiple rows.
 
 Each argument is an arrayref representing a row.
 
-Any argument that is not a reference is taken to be the title of a new worksheet.
+Any argument that is not a reference (i.e. a scalar) is taken to be the
+title of a new worksheet.
 
 =cut
 
-sub addrows (@)
-{
-	my $self = shift;
-	foreach my $row (@_)
-	{
-		if (ref $row eq 'ARRAY')
-		{
+sub addrows (@) {
+	my $self=shift;
+
+	foreach my $row (@_) {
+		if (ref $row eq 'ARRAY') {
 			$self->addrow(@$row);
 		}
-		elsif (!ref $row)
-		{
+		elsif (!ref $row) {
 			$self->addsheet($row);
 		}
-		else
-		{
-			warn "Could not add row.";
+		else {
+			warn "Unsupported row format '".ref($row)."'";
 		}
 	}
+
 	return $self;
 }
 
-
 ###############################################################################
 
 =head2 addsheet(name)
@@ -322,42 +351,53 @@ currently.
 
 =cut
 
-sub addsheet ($$)
-{
-	die "addsheet not implemented!!\n";
+sub addsheet ($$) {
+    die (((caller(0))[3])." - pure virtual method called");
 }
 
 ###############################################################################
 
-=head2 freeze($row, $col, $top_row, $left_col)
+=head2 freeze($row, $col, $top_row, $left_col))
 
 Sets a freeze-pane at the given position, equivalent to Spreadsheet::WriteExcel->freeze_panes().
-Only implemented for Excel spreadsheets so far.
+Ignored for CSV files.
 
 =cut
 
-sub freeze {}
+sub freeze (@) {
+    return shift;
+}
 
 ###############################################################################
 
-=head2 close()
+=head2 close ()
 
-Saves the spreadsheet to disk (some of the modules save incrementally anyway) and
-closes the file. Calling this explicitly is usually un-necessary, as the Perl garbage collector
-will do the job eventually anyway. Once a spreadsheet is closed, calls to addrow() will
-fail.
+Finalizes the spreadsheet and closes the file. It is a good idea
+to call this method explicitly instead of relying on perl's garbage
+collector because for many formats the file may be in an unusable
+state until this method is called.
+
+Once a spreadsheet is closed, calls to addrow() will fail.
 
 =cut
 
-sub close {}
+sub close {
+    ### print STDERR "0: ".join("/",map { $_ || 'UNDEF' } caller(0))."\n";
+    ### print STDERR "1: ".join("/",map { $_ || 'UNDEF' } caller(1))."\n";
+    ### print STDERR "2: ".join("/",map { $_ || 'UNDEF' } caller(2))."\n";
+    ### print STDERR "3: ".join("/",map { $_ || 'UNDEF' } caller(3))."\n";
+    die (((caller(0))[3])." - pure virtual method called");
+}
+
+###############################################################################
 
 1;
 __END__
 
 =head1 AUTHORS
 
-Versions 0.01 to 0.03 by Nick Eremeev <nick.eremeev@gmail.com> L<http://ejelta.com/>.
-
-Toby Inkster <tobyink@cpan.org> has taken over maintenance of this package, but
-attempts to contact the original author have so far gone unreplied.
+Written by Nick Eremeev <nick.eremeev@gmail.com>; Andrew Maltsev <am@ejelta.com>;
+http://ejelta.com/
 
+Multiple backends implementation and other patches by Toby Inkster
+<tobyink@cpan.org> (see also a full fork at L<Spreadsheet::Wright>).
